(function () {

    var _addProp = function () {
        var self = this;
        var length = arguments.length;
        if (!length) return;
        for (var i = 0; i < length; i++) {
            var props = arguments[i];
            var propNames = Object.keys(props);
            propNames.forEach(function (name) {
                var desc = Object.getOwnPropertyDescriptor(props, name);
                Object.defineProperty(self, name, desc);
            });
        }
    };

    var Class = function () {
        _addProp.apply(this, arguments);
        this.init && this.init.apply(this, arguments);
    };

    var arrType = [
        'Object',
        'Array',
        'Function',
        'Arguments',
        'String',
        'RegExp',
        'Number',
        'Boolean',
        'Undefined',
        'Null',
        'NaN'
    ];

    var typeOf = Class.typeOf = function (obj) {
        var _toString = Object.prototype.toString;
        var result = 'unknown';
        arrType.forEach(function (type) {
            if (_toString.call(obj) === '[object ' + type + ']') result = type.toLowerCase();
            if (result === 'number' && isNaN(obj)) result = 'NaN';
        });
        return result;
    };

    var is = {};
    arrType.forEach(function (type) {
        is[type] = Class['is' + type] = function (obj) {
            return typeOf(obj).toLowerCase() === type.toLowerCase();
        }
    });

    var has = Class.has = Class.prototype.has = function (prop) {
        var self = is.Function(this) ? this.prototype : this;
        return typeof self[prop] !== 'undefined' && !is.Null(self[prop]);
    };

    var init = Class.prototype.init = function () {
        return this;
    };

    var extend = Class.extend = Class.prototype.extend = function () {
        var isConstructor = is.Function(this);
        var Parent = isConstructor ? this : this.constructor;
        var Child = function () {
            Parent.apply(this, arguments);
        };
        Child.prototype = Object.create(Parent.prototype);
        Child.prototype.constructor = Child;
        if (!isConstructor) arguments[arguments.length] = this;
        _addProp.apply(Child.prototype, arguments);
        _addProp.call(Child, Parent);
        return isConstructor ? Child : new Child();
    };

    var copyOf = Class.copyOf = function (obj) {
        var copy = Object.create(Object.getPrototypeOf(obj));
        var propNames = Object.getOwnPropertyNames(obj);
        propNames.forEach(function (name) {
            var desc = Object.getOwnPropertyDescriptor(obj, name);
            Object.defineProperty(copy, name, desc);
        });
        return copy;
    };

    var global = this;
    if (typeof exports !== 'undefined') {
        if (typeof module !== 'undefined' && module.exports) {
            exports = module.exports = Class;
        }
        exports.Class = Class;
    } else {
        global.Class = Class;
    }
}).call(this);